@@ -28,6 +28,7 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtklabel.h>
@@ -144,7 +145,7 @@ connect_to_server (NautilusConnectServerDialog *dialog)
 		}
 	} else {
 		char *method, *user, *port, *initial_path, *server, *folder;
-		char *t;
+		char *t, *join;
 		gboolean free_initial_path, free_user, free_port;
 
 		server = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->server_entry), 0, -1);
@@ -196,23 +197,36 @@ connect_to_server (NautilusConnectServerDialog *dialog)
 		folder = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->folder_entry), 0, -1);
 		if (dialog->details->user_entry->parent != NULL) {
 			free_user = TRUE;
-			user = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->user_entry), 0, -1);
+			
+			t = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->user_entry), 0, -1);
+
+			user = gnome_vfs_escape_string (t);
+
+			g_free (t);
 		}
 
 		if (folder[0] != 0 &&
 		    folder[0] != '/') {
-			t = folder;
-			folder = g_strconcat ("/", t, NULL);
-			g_free (t);
+			join = "/";
+		} else {
+			join = "";
 		}
-		
-		uri = g_strdup_printf ("%s://%s%s%s%s%s%s%s",
+
+		t = folder;
+		folder = g_strconcat (initial_path, join, t, NULL);
+		g_free (t);
+
+		t = folder;
+		folder = gnome_vfs_escape_path_string (t);
+		g_free (t);
+
+		uri = g_strdup_printf ("%s://%s%s%s%s%s%s",
 				       method,
 				       user, (user[0] != 0) ? "@" : "",
 				       server,
 				       (port[0] != 0) ? ":" : "", port,
-				       initial_path,
 				       folder);
+
 		if (free_initial_path) {
 			g_free (initial_path);
 		}
@@ -536,10 +550,6 @@ setup_for_type (NautilusConnectServerDialog *dialog)
 
  connection_name:
 	
-	gtk_table_set_row_spacing  (GTK_TABLE (table),
-				    i-1,
-				    18);
-	
 	label = gtk_label_new_with_mnemonic (_("_Name to use for connection:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
@@ -594,18 +604,24 @@ nautilus_connect_server_dialog_init (NautilusConnectServerDialog *dialog)
 	GtkWidget *table;
 	GtkWidget *combo;
 	GtkWidget *hbox;
+	GtkWidget *vbox;
 	
 	dialog->details = g_new0 (NautilusConnectServerDialogDetails, 1);
 
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Connect to Server"));
-	gtk_window_set_default_size (GTK_WINDOW (dialog), 300, -1);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 
-
-	hbox = gtk_hbox_new (FALSE, 6);
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+			    vbox, FALSE, TRUE, 0);
+	gtk_widget_show (vbox);
+			    
+	hbox = gtk_hbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (vbox),
 			    hbox, FALSE, TRUE, 0);
 	gtk_widget_show (hbox);
 	
@@ -635,25 +651,24 @@ nautilus_connect_server_dialog_init (NautilusConnectServerDialog *dialog)
 	gtk_widget_show (combo);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
 	gtk_box_pack_start (GTK_BOX (hbox),
-			    combo, FALSE, TRUE, 0);
+			    combo, TRUE, TRUE, 0);
 	g_signal_connect (combo, "changed",
 			  G_CALLBACK (combo_changed_callback),
 			  dialog);
 	
 
 	hbox = gtk_hbox_new (FALSE, 6);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (vbox),
 			    hbox, FALSE, TRUE, 0);
 	gtk_widget_show (hbox);
 
-	label = gtk_label_new_with_mnemonic ("       ");
+	label = gtk_label_new_with_mnemonic ("    ");
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox),
 			    label, FALSE, FALSE, 0);
 	
 	
 	dialog->details->table = table = gtk_table_new (5, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 5);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
 	gtk_widget_show (table);
